@@ -2,12 +2,13 @@
 
 from __future__ import unicode_literals
 
+import uuid
+
 from django.core.validators import MaxValueValidator, MinValueValidator
 from django.db import models, transaction
 from django.utils.encoding import force_text
 from django.utils.text import slugify
 from django.utils.translation import gettext_lazy as _
-
 from six import python_2_unicode_compatible
 
 from . import classproperty
@@ -20,7 +21,6 @@ from .utils import join_pks, split_pks
 
 @python_2_unicode_compatible
 class TreeNodeModel(models.Model):
-
     """
     Usage:
 
@@ -85,10 +85,10 @@ class TreeNodeModel(models.Model):
         verbose_name=_('Level'), )
 
     tn_parent = models.ForeignKey('self',
-        related_name='tn_children',
-        on_delete=models.CASCADE,
-        blank=True, null=True,
-        verbose_name=_('Parent'), )
+                                  related_name='tn_children',
+                                  on_delete=models.CASCADE,
+                                  blank=True, null=True,
+                                  verbose_name=_('Parent'), )
 
     tn_priority = models.PositiveSmallIntegerField(
         default=0,
@@ -131,9 +131,12 @@ class TreeNodeModel(models.Model):
     def get_ancestors_count(self):
         return self.tn_ancestors_count
 
+    def get_ancestors_pks(self):
+        return split_pks(self.tn_ancestors_pks)
+
     def get_ancestors_queryset(self):
         return self.__class__.objects.filter(
-            pk__in=split_pks(self.tn_ancestors_pks))
+            pk__in=self.get_ancestors_pks())
 
     def get_breadcrumbs(self, attr=None, cache=True):
         objs = self.get_ancestors(cache=cache) + [self]
@@ -148,9 +151,12 @@ class TreeNodeModel(models.Model):
     def get_children_count(self):
         return self.tn_children_count
 
+    def get_children_pks(self):
+        return split_pks(self.tn_children_pks)
+
     def get_children_queryset(self):
         return self.__class__.objects.filter(
-            pk__in=split_pks(self.tn_children_pks))
+            pk__in=self.get_children_pks())
 
     def get_depth(self):
         return self.tn_depth
@@ -164,16 +170,19 @@ class TreeNodeModel(models.Model):
     def get_descendants_count(self):
         return self.tn_descendants_count
 
+    def get_descendants_pks(self):
+        return split_pks(self.tn_descendants_pks)
+
     def get_descendants_queryset(self):
         return self.__class__.objects.filter(
-            pk__in=split_pks(self.tn_descendants_pks))
+            pk__in=self.get_descendants_pks())
 
     def get_descendants_tree(self, cache=True):
         return self.__get_nodes_tree(instance=self, cache=cache)
 
     def get_descendants_tree_display(self, cache=True):
         objs = self.get_descendants(cache=cache)
-        strs = ['%s' % (obj, ) for obj in objs]
+        strs = ['%s' % (obj,) for obj in objs]
         d = '\n'.join(strs)
         return d
 
@@ -223,6 +232,9 @@ class TreeNodeModel(models.Model):
     def get_parent(self):
         return self.tn_parent
 
+    def get_parent_pk(self):
+        return self.tn_parent_id
+
     def set_parent(self, obj):
         with no_signals():
             if obj:
@@ -230,9 +242,9 @@ class TreeNodeModel(models.Model):
                 cls = self.__class__
                 if obj_cls != cls:
                     raise ValueError(
-                        'obj can\'t be set as parent, '\
+                        'obj can\'t be set as parent, ' \
                         'it is istance of %s, expected instance of %s.' % (
-                        obj_cls.__name__, cls.__name__, ))
+                            obj_cls.__name__, cls.__name__,))
                 if obj == self:
                     raise ValueError(
                         'obj can\'t be set as parent of itself.')
@@ -253,18 +265,21 @@ class TreeNodeModel(models.Model):
         self.save()
 
     def get_root(self, cache=True):
-        root_pk = (split_pks(self.tn_ancestors_pks) + [self.pk])[0]
+        root_pk = self.get_root_pk()
         if cache:
             root_obj = query_cache(self.__class__, pk=root_pk)
         else:
             root_obj = self.__class__.objects.get(pk=root_pk)
         return root_obj
 
+    def get_root_pk(self):
+        return (split_pks(self.tn_ancestors_pks) + [self.pk])[0]
+
     @classmethod
     def get_roots(cls, cache=True):
         if cache:
             return [obj for obj in query_cache(cls) \
-                if obj.tn_ancestors_count == 0]
+                    if obj.tn_ancestors_count == 0]
         else:
             return list(cls.get_roots_queryset())
 
@@ -281,9 +296,12 @@ class TreeNodeModel(models.Model):
     def get_siblings_count(self):
         return self.tn_siblings_count
 
+    def get_siblings_pks(self):
+        return split_pks(self.tn_siblings_pks)
+
     def get_siblings_queryset(self):
         return self.__class__.objects.filter(
-            pk__in=split_pks(self.tn_siblings_pks))
+            pk__in=self.get_siblings_pks())
 
     @classmethod
     def get_tree(cls, cache=True):
@@ -295,7 +313,7 @@ class TreeNodeModel(models.Model):
             objs = query_cache(cls)
         else:
             objs = list(cls.objects.all())
-        strs = ['%s' % (obj, ) for obj in objs]
+        strs = ['%s' % (obj,) for obj in objs]
         d = '\n'.join(strs)
         return d
 
@@ -313,19 +331,19 @@ class TreeNodeModel(models.Model):
         return (self.__class__ == obj.__class__ and \
                 self.pk and \
                 self.pk != obj.pk and \
-                self.pk in split_pks(obj.tn_ancestors_pks))
+                str(self.pk) in split_pks(obj.tn_ancestors_pks))
 
     def is_child_of(self, obj):
         return (self.__class__ == obj.__class__ and \
                 self.pk and \
                 self.pk != obj.pk and \
-                self.pk in split_pks(obj.tn_children_pks))
+                str(self.pk) in split_pks(obj.tn_children_pks))
 
     def is_descendant_of(self, obj):
         return (self.__class__ == obj.__class__ and \
                 self.pk and \
                 self.pk != obj.pk and \
-                self.pk in split_pks(obj.tn_descendants_pks))
+                str(self.pk) in split_pks(obj.tn_descendants_pks))
 
     def is_first_child(self):
         return (self.pk and \
@@ -344,7 +362,7 @@ class TreeNodeModel(models.Model):
                 self.pk and \
                 self.pk != obj.pk and \
                 obj.tn_ancestors_count > 0 and \
-                self.pk == split_pks(obj.tn_ancestors_pks)[-1])
+                str(self.pk) == split_pks(obj.tn_ancestors_pks)[-1])
 
     def is_root(self):
         return (self.pk and \
@@ -363,16 +381,15 @@ class TreeNodeModel(models.Model):
     def update_tree(cls):
 
         debug_message_prefix = '[treenode] update %s.%s tree: ' % (
-            cls.__module__, cls.__name__, )
+            cls.__module__, cls.__name__,)
 
         with debug_performance(debug_message_prefix):
-
             # update db
             objs_data = cls.__get_nodes_data()
 
             with transaction.atomic():
                 for obj_key, obj_data in objs_data.items():
-                    obj_pk = int(obj_key)
+                    obj_pk = obj_key
                     cls.objects.filter(pk=obj_pk).update(**obj_data)
 
             # update in-memory instances
@@ -391,9 +408,15 @@ class TreeNodeModel(models.Model):
         alphabetical_val = slugify(str(self))
         alphabetical_key = alphabetical_val.ljust(priority_len, str('z'))
         alphabetical_key = alphabetical_key[0:priority_len]
-        pk_val = min(self.pk, priority_max)
+
+        if isinstance(self.pk, uuid.UUID):
+            pk_val = self.pk.int
+            pk_val = int(str(pk_val)[:priority_len])
+        else:
+            pk_val = min(self.pk, priority_max)
+
         pk_key = str(pk_val).zfill(priority_len)
-        s = '%s%s%s' % (priority_key, alphabetical_key, pk_key, )
+        s = '%s%s%s' % (priority_key, alphabetical_key, pk_key,)
         s = s.upper()
         return s
 
@@ -402,14 +425,14 @@ class TreeNodeModel(models.Model):
         obj_dict = {}
 
         # update ancestors
-        parent_pk = self.tn_parent_id
+        parent_pk = self.get_parent_pk()
 
         ancestors_list = []
         ancestor_pk = parent_pk
         while ancestor_pk:
             ancestor_obj = objs_dict.get(str(ancestor_pk))
             ancestors_list.insert(0, ancestor_obj)
-            ancestor_pk = ancestor_obj.tn_parent_id
+            ancestor_pk = ancestor_obj.get_parent_pk()
         ancestors_pks = [obj.pk for obj in ancestors_list]
         ancestors_count = len(ancestors_pks)
 
@@ -442,8 +465,8 @@ class TreeNodeModel(models.Model):
 
         objs_qs = cls.objects.select_related('tn_parent')
         objs_list = list(objs_qs)
-        objs_dict = {str(obj.pk):obj for obj in objs_list}
-        objs_data_dict = {str(obj.pk):obj.__get_node_data(objs_list, objs_dict) for obj in objs_list}
+        objs_dict = {str(obj.pk): obj for obj in objs_list}
+        objs_data_dict = {str(obj.pk): obj.__get_node_data(objs_list, objs_dict) for obj in objs_list}
         objs_data_sort = lambda obj: objs_data_dict[str(obj['pk'])]['tn_order_str']
         objs_data_list = list(objs_data_dict.values())
         objs_data_list.sort(key=objs_data_sort)
@@ -566,7 +589,7 @@ class TreeNodeModel(models.Model):
     def __get_nodes_tree(cls, instance=None, cache=True):
 
         def __get_node_tree(obj):
-            child_tree = { 'node':obj, 'tree':[] }
+            child_tree = {'node': obj, 'tree': []}
             if obj.tn_children_pks:
                 children_pks = split_pks(obj.tn_children_pks)
                 for child_pk in children_pks:
@@ -583,20 +606,20 @@ class TreeNodeModel(models.Model):
                 objs_list = query_cache(cls, pks=objs_pks)
             else:
                 objs_list = list(cls.objects.filter(pk__in=split_pks(objs_pks)))
-            objs_dict = { str(obj.pk):obj for obj in objs_list }
+            objs_dict = {str(obj.pk): obj for obj in objs_list}
             objs_tree = __get_node_tree(instance)['tree']
         else:
             if cache:
                 objs_list = query_cache(cls)
             else:
                 objs_list = list(cls.objects.all())
-            objs_dict = { str(obj.pk):obj for obj in objs_list }
+            objs_dict = {str(obj.pk): obj for obj in objs_list}
             objs_tree = [__get_node_tree(obj)
-                        for obj in objs_list if obj.tn_level == 1]
+                         for obj in objs_list if obj.tn_level == 1]
 
         return objs_tree
 
-    # Public properties
+    #  Public properties
     # All properties map a get_{{property}}() method.
 
     @property
@@ -607,6 +630,10 @@ class TreeNodeModel(models.Model):
     def ancestors_count(self):
         return self.get_ancestors_count()
 
+    @property
+    def ancestors_pks(self):
+        return self.get_ancestors_pks()
+
     @property
     def breadcrumbs(self):
         return self.get_breadcrumbs()
@@ -619,6 +646,10 @@ class TreeNodeModel(models.Model):
     def children_count(self):
         return self.get_children_count()
 
+    @property
+    def children_pks(self):
+        return self.get_children_pks()
+
     @property
     def depth(self):
         return self.get_depth()
@@ -631,6 +662,10 @@ class TreeNodeModel(models.Model):
     def descendants_count(self):
         return self.get_descendants_count()
 
+    @property
+    def descendants_pks(self):
+        return self.get_descendants_pks()
+
     @property
     def descendants_tree(self):
         return self.get_descendants_tree()
@@ -663,6 +698,10 @@ class TreeNodeModel(models.Model):
     def parent(self):
         return self.get_parent()
 
+    @property
+    def parent_pk(self):
+        return self.get_parent_pk()
+
     @property
     def priority(self):
         return self.get_priority()
@@ -675,6 +714,10 @@ class TreeNodeModel(models.Model):
     def root(self):
         return self.get_root()
 
+    @property
+    def root_pk(self):
+        return self.get_root_pk()
+
     @property
     def siblings(self):
         return self.get_siblings()
@@ -683,6 +726,10 @@ class TreeNodeModel(models.Model):
     def siblings_count(self):
         return self.get_siblings_count()
 
+    @property
+    def siblings_pks(self):
+        return self.get_siblings_pks()
+
     @classproperty
     def tree(cls):
         return cls.get_tree()
@@ -698,4 +745,5 @@ class TreeNodeModel(models.Model):
     def __str__(self):
         return self.get_display(indent=True)
 
+
 connect_signals()
